import { Entity, Player, system, world } from "@minecraft/server";
import { ModalFormData } from "@minecraft/server-ui";
import cannon from "mobs/cannon";
import platform from "mobs/platform";
import player from "mobs/player";
import rocket from "mobs/rocket";
import spring from "mobs/spring";
import { point } from "point";
import { util } from "util";
globalThis.accessible = accessible;
globalThis.getEntities = getEntities;
globalThis.point = point;
Entity.prototype.getComp = function(mobIds) {
    const comp = comps[this.id];
    return comp;
};
Entity.prototype.loc = function() {
    return point.fromVector3(this.location);
};
Entity.prototype.rot = function() {
    return point.fromVector3(this.getViewDirection());
};
Player.prototype.getHeldItem = function() {
    return util.selectedItem(this);
};
Player.prototype.setHeldItem = function(item) {
    util.setSelectedSlot(this, item);
};
let isWorldInit = false;
world.afterEvents.playerSpawn.subscribe(()=>{
    if (isWorldInit) return;
    isWorldInit = true;
});
export function accessible(entity) {
    try {
        // @ts-ignore
        return entity?.isValid();
    } catch (e) {
        return false;
    }
}
export const propertyDefaults = {
    ammo: 0,
    targetId: "",
    origin: "",
    destination: "",
    hasTakenOff: false,
    hasForced3rdCamera: false,
    power: 1,
    freeRidingTime: 0
};
export function getProperty(entity, property) {
    const val = entity.getDynamicProperty(property) ?? propertyDefaults[property];
    return val;
}
export function setProperty(entity, property, value) {
    entity.setDynamicProperty(property, value);
}
export const comps = {};
const regs = new Set();
const compDefs = {
    "minecraft:player": player,
    "ns_mvt:spring": spring,
    "ns_mvt:cannon": cannon,
    "ns_mvt:rocket": rocket,
    "ns_mvt:platform": platform
};
world.afterEvents.playerSpawn.subscribe(({ player })=>init(player));
world.afterEvents.entitySpawn.subscribe(({ entity })=>init(entity));
world.afterEvents.entityLoad.subscribe(({ entity })=>init(entity));
[
    ...world.getDimension("overworld").getEntities(),
    ...world.getDimension("nether").getEntities(),
    ...world.getDimension("the_end").getEntities()
].forEach((entity)=>{
    if (entity.typeId == "ns_mvt:tutorial") entity.remove();
    else init(entity);
});
system.runInterval(()=>cleanup(), 40);
world.beforeEvents.playerInteractWithBlock.subscribe((eventData)=>{
    if (!eventData.isFirstEvent) return;
    system.run(()=>{
        if (eventData.itemStack?.typeId !== "ns_mvt:rocket") return;
        eventData.player.onScreenDisplay.setActionBar("§cRockets must be placed on §iRocket Platforms§r");
    });
});
function init(entity) {
    if (!accessible(entity)) return;
    if (regs.has(entity.id)) return;
    regs.add(entity.id);
    for(const key in compDefs){
        if (entity.typeId == key) {
            comps[entity.id] = new compDefs[key](entity);
            return;
        }
    }
}
export function getEntities(query) {
    return [
        ...new Set([
            ...world.getDimension("overworld").getEntities(query),
            ...world.getDimension("nether").getEntities(query),
            ...world.getDimension("the_end").getEntities(query)
        ])
    ];
}
function cleanup() {
    for(const entityId in comps){
        const entity = world.getEntity(entityId);
        if (!accessible(entity)) {
            comps[entityId].remove();
            delete comps[entityId];
            regs.delete(entityId);
        }
    }
}
world.afterEvents.playerInteractWithEntity.subscribe((eventData)=>{
    for(const entityId in comps){
        const entity = world.getEntity(entityId);
        if (!accessible(entity)) continue;
        if (entity.id == eventData.target.id) {
            comps[entityId].onInteract(eventData.player);
            return;
        }
    }
});
world.afterEvents.itemStartUse.subscribe((eventData)=>{
    if (eventData.itemStack.typeId !== "ns_mvt:tutorial") return;
    const oldEntities = getEntities({
        type: "ns_mvt:tutorial",
        tags: [
            eventData.source.name
        ]
    });
    for (const entity of oldEntities){
        entity.remove();
    }
    const spawnLocation = new point(eventData.source.location.x + 1, -64, eventData.source.location.z);
    const newEntity = eventData.source.dimension.spawnEntity("ns_mvt:tutorial", spawnLocation);
    newEntity.addTag(eventData.source.name);
    eventData.source.dimension.playSound("item.book.page_turn", eventData.source.location);
    system.runTimeout(()=>{
        if (!accessible(eventData.source)) return;
        const cmd = `dialogue open @e[type=ns_mvt:tutorial,x=${spawnLocation.x},y=${spawnLocation.y},z=${spawnLocation.z},c=1] @s ns_mvt:tutorial`;
        eventData.source.runCommand(cmd);
    }, 4);
});
function onTick() {
    // const entities = getEntities({ excludeFamilies: ["player"] });
    // for (const entity of entities) {
    //     if (entity.typeId !== "minecraft:player") {
    //         entity.addEffect("slowness", 40, { amplifier: 20, showParticles: false });
    //         entity.teleport(entity.loc(), { facingLocation: point.fromString("-246 76 623") });
    //     }
    // }
    return;
}
system.runInterval(onTick, 1);
const allIds = world.getDynamicPropertyIds();
if (!allIds.includes("configDoubleJump")) world.setDynamicProperty("configDoubleJump", true);
if (!allIds.includes("configDash")) world.setDynamicProperty("configDash", true);
if (!allIds.includes("configClimb")) world.setDynamicProperty("configClimb", true);
if (!allIds.includes("configWaterRun")) world.setDynamicProperty("configWaterRun", true);
if (!allIds.includes("configWallRun")) world.setDynamicProperty("configWallRun", true);
if (!allIds.includes("configWallJump")) world.setDynamicProperty("configWallJump", true);
if (!allIds.includes("configRoll")) world.setDynamicProperty("configRoll", true);
if (!allIds.includes("configAbilitiesConsumeHunger")) world.setDynamicProperty("configAbilitiesConsumeHunger", true);
const configCache = {
    DoubleJump: world.getDynamicProperty("configDoubleJump"),
    Dash: world.getDynamicProperty("configDash"),
    Climb: world.getDynamicProperty("configClimb"),
    WaterRun: world.getDynamicProperty("configWaterRun"),
    WallRun: world.getDynamicProperty("configWallRun"),
    WallJump: world.getDynamicProperty("configWallJump"),
    Roll: world.getDynamicProperty("configRoll"),
    AbilitiesConsumeHunger: world.getDynamicProperty("configAbilitiesConsumeHunger")
};
export function isConfigEnabled(config) {
    return configCache[config];
}
system.afterEvents.scriptEventReceive.subscribe((eventData)=>{
    const message = eventData.id;
    if (!message.startsWith("ns_mvt:")) return;
    const player = eventData.initiator;
    const npc = eventData.sourceEntity;
    let id;
    if (message == "ns_mvt:tutorial") {
        npc.playAnimation("animation.ninja_squirrel_mvt.spring_reverse", {
            controller: "spring"
        });
        id = 0;
    } else if (message == "ns_mvt:skills") {
        npc.playAnimation("animation.ninja_squirrel_mvt.spring", {
            controller: "spring"
        });
        id = 0;
    } else if (message == "ns_mvt:entities") {
        id = 1;
    } else if (message == "ns_mvt:gunpowder") {
        id = 2;
    } else if (message == "ns_mvt:options") {
        system.runTimeout(()=>{
            const form = new ModalFormData();
            form.title("Options");
            form.toggle("Enable Double Jump", world.getDynamicProperty("configDoubleJump"));
            form.toggle("Enable Dash", world.getDynamicProperty("configDash"));
            form.toggle("Enable Climb", world.getDynamicProperty("configClimb"));
            form.toggle("Enable Water Run", world.getDynamicProperty("configWaterRun"));
            form.toggle("Enable Wall Run", world.getDynamicProperty("configWallRun"));
            form.toggle("Enable Wall Jump", world.getDynamicProperty("configWallJump"));
            form.toggle("Enable Roll", world.getDynamicProperty("configRoll"));
            form.toggle("Abilities Consume Hunger", world.getDynamicProperty("configAbilitiesConsumeHunger"));
            form.show(player).then((data)=>{
                if (!data.formValues) return;
                world.setDynamicProperty("configDoubleJump", data.formValues[0]);
                world.setDynamicProperty("configDash", data.formValues[1]);
                world.setDynamicProperty("configClimb", data.formValues[2]);
                world.setDynamicProperty("configWaterRun", data.formValues[3]);
                world.setDynamicProperty("configWallRun", data.formValues[4]);
                world.setDynamicProperty("configWallJump", data.formValues[5]);
                world.setDynamicProperty("configRoll", data.formValues[6]);
                world.setDynamicProperty("configAbilitiesConsumeHunger", data.formValues[7]);
                configCache.DoubleJump = data.formValues[0];
                configCache.Dash = data.formValues[1];
                configCache.Climb = data.formValues[2];
                configCache.WaterRun = data.formValues[3];
                configCache.WallRun = data.formValues[4];
                configCache.WallJump = data.formValues[5];
                configCache.Roll = data.formValues[6];
                configCache.AbilitiesConsumeHunger = data.formValues[7];
            });
        }, 10);
        return;
    } else return;
    const cmd = `dialogue open @e[type=ns_mvt:tutorial,x=${npc.location.x},y=${npc.location.y},z=${npc.location.z},c=1] @s ${message}`;
    player.runCommand(cmd);
    player.playSound("item.book.page_turn");
    npc.setProperty("ns_mvt:type", id);
});
