import { system } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";
import { util } from "util";
import mob from "./mob";
class cannon extends mob {
    onTick() {
        const rider = this.getRider();
        this.rideTicks = rider ? this.rideTicks + 1 : 0;
        if (this.rideTicks == 2) {
            this.queueAnimation("animation.ninja_squirrel_mvt.cannon.shoot", {
                controller: "shoot",
                nextState: "default2"
            });
            rider.getComp("minecraft:player").queueAnimation("animation.ninja_squirrel_mvt.hide", {
                controller: "hide"
            });
            rider.getComp("minecraft:player").force3rdPersonCamera();
        }
        if (this.rideTicks > 21) {
            system.runTimeout(()=>{
                if (!accessible(rider)) return;
                this.throw(rider);
            }, 1);
            system.runTimeout(()=>{
                if (!accessible(rider)) return;
                rider.getComp("minecraft:player").queueAnimation("animation.ninja_squirrel_mvt.jump2", {
                    controller: "mvt",
                    blendOutTime: 0.2,
                    nextState: "animation.ninja_squirrel_mvt.double_jump",
                    stopExpression: "q.is_on_ground"
                });
            }, 2);
            this.rideTicks = 0;
        }
    }
    throw(rider) {
        rider.teleport(this.mob.loc().add(0, 1, 0));
        const xRot = this.mob.getProperty("ns_mvt:rot_x");
        const yRot = 90 - this.mob.getProperty("ns_mvt:rot_y");
        const dir = point.fromRotation(xRot, yRot);
        const xz = dir.flat().length();
        const power = this.getProperty("power");
        let rideDuration = [
            3,
            5,
            9,
            14,
            25
        ][power - 1];
        const xzPower = [
            2,
            3,
            3.3,
            3.6,
            4
        ][power - 1];
        const yPower = [
            1,
            1.5,
            1.7,
            1.9,
            2.1
        ][power - 1];
        const applyKnockback = ()=>{
            rider.getComp("minecraft:player").landedFromCannon = false;
            rider.applyKnockback(dir.x, dir.z, xz * xzPower, dir.y * yPower);
            rideDuration--;
            if (rideDuration > 0) {
                system.runTimeout(applyKnockback, 1);
            }
        };
        applyKnockback();
        this.playSfx("ns_mvt.cannon", 0.4, 1.4);
    }
    onInteract(player) {
        const form = new ActionFormData();
        form.title("Cannon");
        form.body("Current gunpowder: " + this.ammo);
        const buttons = [];
        buttons.push(`Ride (${this.getProperty("power") - 1} gunpowder)`);
        buttons.push("Insert Gunpowder");
        buttons.push("Set Rotation");
        buttons.push("Set Power");
        buttons.push("Pick Up");
        for (const button of buttons){
            form.button(button);
        }
        form.show(player).then((data)=>{
            if (typeof data.selection != "number") return;
            if (!accessible(this.mob)) return;
            switch(buttons[data.selection]){
                case `Ride (${this.getProperty("power") - 1} gunpowder)`:
                    this.ride(player);
                    break;
                case "Pick Up":
                    this.pickUp();
                    break;
                case "Set Power":
                    this.setPower(player);
                    break;
                case "Insert Gunpowder":
                    this.insertAmmo(player);
                    break;
                case "Set Rotation":
                    this.setDirection(player);
                    break;
            }
        });
    }
    insertAmmo(player) {
        const ammoInInventoryCount = util.getItemCount(player, "minecraft:gunpowder");
        if (ammoInInventoryCount == 0) {
            player.sendMessage("You need gunpowder in your inventory.");
            return;
        }
        const form = new ModalFormData();
        form.title("Insert Gunpowder");
        form.slider("Amount", 1, ammoInInventoryCount, 1);
        form.show(player).then((data)=>{
            if (!data.formValues) return;
            if (!accessible(this.mob)) return;
            const amount = data.formValues[0];
            const ammoInInventory = util.getItems(player, "minecraft:gunpowder");
            let itemsCountToRemove = amount;
            for (const { itemStack, slot } of ammoInInventory){
                if (itemsCountToRemove <= 0) break;
                if (itemStack.amount <= itemsCountToRemove) {
                    util.setInventoryItem(player, undefined, slot);
                    itemsCountToRemove -= itemStack.amount;
                } else {
                    itemStack.amount -= itemsCountToRemove;
                    util.setInventoryItem(player, itemStack, slot);
                    itemsCountToRemove = 0;
                }
            }
            this.ammo += amount;
        });
    }
    setPower(player) {
        const power = this.getProperty("power");
        const form = new ModalFormData();
        form.title("Set Power");
        form.slider("Power", 1, 5, 1, power);
        form.show(player).then((data)=>{
            if (!data.formValues) return;
            if (!accessible(this.mob)) return;
            const power = data.formValues[0];
            this.setProperty("power", power);
        });
    }
    setDirection(player) {
        const form = new ModalFormData();
        form.title("Set Rotation");
        form.slider("X Rotation", 0, 45, 1, this.mob.getProperty("ns_mvt:rot_x"));
        form.slider("Y Rotation", 0, 360, 1, this.mob.getProperty("ns_mvt:rot_y"));
        form.show(player).then((data)=>{
            if (!data.formValues) return;
            if (!accessible(this.mob)) return;
            const x = data.formValues[0];
            const y = data.formValues[1];
            this.mob.setProperty("ns_mvt:rot_x", x);
            this.mob.setProperty("ns_mvt:rot_y", y);
        });
    }
    ride(player) {
        const hasRider = this.getRider();
        if (hasRider) return;
        if (this.ammo < this.getProperty("power") - 1) {
            player.sendMessage("Not enough gunpowder.");
            return;
        }
        this.ammo = this.ammo - this.getProperty("power") + 1;
        this.mob.getComponent("rideable")?.addRider(player);
    }
    dejectRiders() {
        this.mob.getComponent("rideable")?.ejectRiders();
    }
    getRider() {
        return this.mob.getComponent("rideable")?.getRiders()[0];
    }
    constructor(entity){
        super(entity);
        this.rideTicks = 0;
        const nearbyCannons = entity.dimension.getEntities({
            type: "ns_mvt:cannon",
            maxDistance: 0.5,
            location: entity.location
        });
        if (nearbyCannons.length > 1) {
            this.pickUp();
        }
    }
}
export { cannon as default };
