import { ItemStack, system } from "@minecraft/server";
import { accessible, getEntities, getProperty as mainGetProperty } from "index";
import { point } from "point";
const AirBlocks = [
    "minecraft:air",
    "minecraft:pink_tulip",
    "minecraft:carrots",
    "minecraft:brown_mushroom",
    "minecraft:large_fern",
    "minecraft:cherry_sapling",
    "minecraft:torchflower",
    "minecraft:cave_vines",
    "minecraft:weeping_vines",
    "minecraft:red_tulip",
    "minecraft:wheat",
    "minecraft:spruce_sapling",
    "minecraft:cave_vines_body_with_berries",
    "minecraft:waterlily",
    "minecraft:sunflower",
    "minecraft:jungle_sapling",
    "minecraft:cave_vines_head_with_berries",
    "minecraft:birch_sapling",
    "minecraft:wither_rose",
    "minecraft:pale_hanging_moss",
    "minecraft:dark_oak_sapling",
    "minecraft:oak_sapling",
    "minecraft:cocoa",
    "minecraft:peony",
    "minecraft:pitcher_crop",
    "minecraft:short_grass",
    "minecraft:pale_oak_sapling",
    "minecraft:chorus_flower",
    "minecraft:reeds",
    "minecraft:allium",
    "minecraft:torchflower_crop",
    "minecraft:acacia_sapling",
    "minecraft:fern",
    "minecraft:deadbush",
    "minecraft:bamboo_sapling",
    "minecraft:azure_bluet",
    "minecraft:oxeye_daisy",
    "minecraft:lilac",
    "minecraft:flowering_azalea",
    "minecraft:poppy",
    "minecraft:cornflower",
    "minecraft:orange_tulip",
    "minecraft:resin_clump",
    "minecraft:azalea",
    "minecraft:nether_wart",
    "minecraft:warped_fungus",
    "minecraft:white_tulip",
    "minecraft:tall_grass",
    "minecraft:rose_bush",
    "minecraft:red_mushroom"
];
const sfx = [
    "ns_mvt.land",
    "ns_mvt.spring",
    "ns_mvt.whoosh1",
    "ns_mvt.whoosh2",
    "ns_mvt.cannon",
    "ns_mvt.rocketstart",
    "ns_mvt.rocketloop"
];
class mob {
    onTick() {}
    getProperty(property) {
        return mainGetProperty(this.mob, property);
    }
    setProperty(property, value) {
        this.mob.setDynamicProperty(property, value);
    }
    get ammo() {
        return this.getProperty("ammo");
    }
    set ammo(value) {
        this.mob.setDynamicProperty("ammo", value);
    }
    runInterval(callback, interval) {
        if (!accessible(this.mob)) return 0;
        const runnerId = system.runInterval(callback, interval);
        this.clearRuns.push(runnerId);
        return runnerId;
    }
    particleLine(from, to) {
        const distance = point.getDistance(from, to);
        const direction = to.subtractVector3(from).normalize();
        const step = 0.5;
        for(let i = 0; i < distance; i += step){
            this.particle(from.addVector3(direction.scale(i)));
        }
    }
    particle(location) {
        // this.mob.dimension.spawnParticle("ns_mvt:soul", point.fromVector3(location));
        this.mob.runCommand(`particlelegacy flame ${location.x} ${location.y} ${location.z}`);
    }
    pickUp() {
        let itemTypeId = this.mob.typeId + "_spawn_egg";
        if (this.mob.typeId === "ns_mvt:rocket") {
            itemTypeId = "ns_mvt:rocket";
        }
        const itemStack = new ItemStack(itemTypeId);
        this.mob.dimension.spawnItem(itemStack, this.mob.location);
        while(this.ammo){
            const ammoMax64 = Math.min(this.ammo, 64);
            const ammoStack = new ItemStack("minecraft:gunpowder", ammoMax64);
            this.mob.dimension.spawnItem(ammoStack, this.mob.location);
            this.ammo -= ammoMax64;
        }
        this.mob.remove();
    }
    runIimeout(callback, delay) {
        if (!accessible(this.mob)) return 0;
        const runnerId = system.runTimeout(callback, delay);
        this.clearRuns.push(runnerId);
        return runnerId;
    }
    playSound(sound) {
        this.mob.dimension.playSound(sound, this.mob.location);
    }
    playSfx(sound, volume = 1, pitch = 1, player) {
        if (player) player.playSound(sound, {
            volume,
            pitch
        });
        else this.mob.dimension.playSound(sound, this.mob.location, {
            volume,
            pitch
        });
    }
    remove() {
        this.clearRuns.forEach((runnerId)=>system.clearRun(runnerId));
        system.clearRun(this.ticker);
    }
    logFinal() {
        if (!this.stuffToLog.length) return;
        getEntities({
            type: "minecraft:player"
        }).forEach((e)=>{
            const msg = this.stuffToLog.map((s)=>{
                if (s.startsWith(",")) {
                    s = s.slice(1);
                }
                return s;
            });
            e.onScreenDisplay.setActionBar(msg.join(""));
        });
        // console.warn(this.stuffToLog.join("\n"));
        this.stuffToLog = [];
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    log(msg) {
        if (!mob.enableLogging) return;
        let str = "";
        if (msg === null) {
            str = "null";
        } else if (Array.isArray(msg)) {
            for (const item of msg){
                str += this.decipherObject(item);
            }
        } else if (typeof msg === "string") {
            str = msg;
        } else if (typeof msg === "number") {
            str = msg.toFixed(2);
        } else if (typeof msg === "boolean") {
            str = msg ? "true" : "false";
        } else if (typeof msg === "undefined") {
            str = "undefined";
        } else if (typeof msg === "function") {
            str = "function";
        } else if (typeof msg === "object") {
            str = this.decipherObject(msg);
        } else if (isNaN(msg)) {
            str = "NaN";
        }
        this.stuffToLog.push(str);
    }
    timeSince(timestamp) {
        return system.currentTick - timestamp;
    }
    queueAnimation(animation, options, skipIfCantPlayInstantly = false, skipIfLastMolangIsSame = false) {
        // console.warn(this.mob.typeId, animation);
        if (skipIfCantPlayInstantly && this.animationQueue.length) {
            return;
        }
        if (skipIfLastMolangIsSame && lastMolang === options?.stopExpression) {
            return;
        }
        this.animationQueue.push({
            animation,
            options
        });
    }
    isSolid(blockTypeId) {
        if (!blockTypeId) return false;
        return !AirBlocks.includes(blockTypeId);
    }
    onInteract(player) {
        return;
    }
    processAnimationQueue() {
        if (this.animationQueue.length) {
            const { animation, options } = this.animationQueue.shift();
            // console.warn(this.mob.typeId, animation);
            this.mob.playAnimation(animation, {
                ...options,
                controller: options?.controller
            });
            // console.warn(animation);
            lastMolang = options?.stopExpression ?? "";
            this.playedAnimationTimestamp.set(animation, system.currentTick);
        // if (this.animationQueue.length >= 2) {
        //     console.warn(this.animationQueue.map((a) => a.animation));
        // }
        }
    }
    timeSincePlayedAnimation(animation) {
        if (!this.playedAnimationTimestamp.has(animation)) return Infinity;
        return system.currentTick - this.playedAnimationTimestamp.get(animation);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    decipherObject(obj) {
        return Object.entries(obj).map(([key, value])=>{
            if (value === null) {
                return `${key}: null\n`;
            } else if (typeof value === "object") {
                if (Array.isArray(value)) {
                    return `${key}: [${value.join("\n")}]\n`;
                }
                if (value.x !== undefined && value.y !== undefined && value.z !== undefined) {
                    return `${key}: (${value.x.toFixed(2)}, ${value.y.toFixed(2)}, ${value.z.toFixed(2)})\n`;
                }
                if (value.x !== undefined && value.y !== undefined) {
                    return `${key}: (${value.x.toFixed(2)}, ${value.y.toFixed(2)})\n`;
                }
                return `${key}:\n${this.decipherObject(value)}\n`;
            } else if (typeof value === "number") {
                return `${key}: ${value.toFixed(2)}\n`;
            }
            return `${key}: ${value}\n`;
        }).join("");
    }
    constructor(entity){
        this.clearRuns = [];
        this.stuffToLog = [];
        this.animationQueue = [];
        this.playedAnimationTimestamp = new Map();
        this.mob = entity;
        this.startLoc = entity.loc();
        this.ticker = this.runInterval(()=>{
            if (!accessible(this.mob)) return;
            this.onTick();
            this.logFinal();
            this.processAnimationQueue();
        }, 1);
    }
}
mob.enableLogging = false;
export { mob as default };
export let lastMolang = "";
