import { system, world } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";
import { getProperty, setProperty } from "index";
import { util } from "util";
import mob from "./mob";
class platform extends mob {
    onInteract(player) {
        const selectedItem = util.selectedItem(player);
        if (selectedItem?.typeId == "ns_mvt:rocket") {
            if (this.rocket) {
                return world.sendMessage("Rocket already exists.");
            }
            this.mob.dimension.spawnEntity("ns_mvt:rocket", this.mob.loc().add(0, 0.2, 0));
            util.removeItem(player, "ns_mvt:rocket", 1);
            return;
        }
        const form = new ActionFormData();
        form.title("Rocket Platform - " + this.mob.nameTag);
        form.body("Current gunpowder: " + (this.rocket?.getComp("ns_mvt:rocket")?.ammo ?? "No Rocket"));
        const buttons = [];
        const rideText = `Travel to ${this.selectedDestination?.entity.nameTag} (${this.selectedDestination?.cost} gunpowder)`;
        if (this.rocket && this.selectedDestination) buttons.push(rideText);
        if (this.rocket) buttons.push("Free Ride");
        if (this.rocket) buttons.push("Insert gunpowder");
        buttons.push("Set name");
        buttons.push("Set destination");
        if (this.rocket) buttons.push("Pick up Rocket");
        buttons.push("Pick Up Platform");
        for (const button of buttons){
            form.button(button);
        }
        form.show(player).then((data)=>{
            if (typeof data.selection != "number") return;
            if (!accessible(this.mob)) return;
            switch(buttons[data.selection]){
                case rideText:
                    this.ride(player);
                    break;
                case "Pick Up Platform":
                    this.pickUpRocket();
                    this.pickUp();
                    break;
                case "Pick up Rocket":
                    this.pickUpRocket();
                    break;
                case "Insert gunpowder":
                    this.insertAmmo(player);
                    break;
                case "Set name":
                    this.setName(player);
                    break;
                case "Set destination":
                    this.setDestination(player);
                    break;
                case "Free Ride":
                    this.freeRide(player);
                    break;
            }
        });
    }
    pickUpRocket() {
        if (!this.rocket) return;
        this.rocket.getComp("ns_mvt:rocket")?.pickUp();
    }
    setDestination(player) {
        const destinations = this.getDestinations();
        if (destinations.length == 0) return player.sendMessage("No destinations found. Place down another platform in the world.");
        const form = new ModalFormData();
        form.title("Set Destination");
        let selectedIndex = destinations.findIndex((d)=>d.entity.id == getProperty(this.mob, "targetId"));
        if (selectedIndex == -1) selectedIndex = 0;
        form.dropdown("Destination", destinations.map((d)=>d.entity.nameTag), selectedIndex);
        form.show(player).then((data)=>{
            if (!data.formValues) return;
            if (!accessible(this.mob)) return;
            const destination = destinations[data.formValues[0]];
            setProperty(this.mob, "targetId", destination.entity.id);
        });
    }
    get rocket() {
        return this.mob.dimension.getEntities({
            closest: 1,
            maxDistance: 2,
            type: "ns_mvt:rocket",
            location: this.mob.location
        })[0];
    }
    getDestinations() {
        const otherPlatforms = this.mob.dimension.getEntities({
            type: "ns_mvt:platform"
        }).filter((e)=>e.id != this.mob.id);
        const destinations = [];
        for (const platform of otherPlatforms){
            const destinationData = this.getDestinationData(platform);
            destinations.push(destinationData);
        }
        return destinations;
    }
    getDestinationData(platform) {
        const distance = this.mob.loc().getDistance(platform.loc());
        const cost = Math.ceil(Math.pow(distance, 0.5));
        // const cost = 0;
        return {
            entity: platform,
            distance,
            cost
        };
    }
    insertAmmo(player) {
        this.rocket?.getComp("ns_mvt:rocket")?.insertAmmo(player);
    }
    setName(player) {
        const form = new ModalFormData();
        form.title("Set Name");
        form.textField("Name", "New name...", this.mob.nameTag);
        form.show(player).then((data)=>{
            if (!data.formValues) return;
            if (!accessible(this.mob)) return;
            const name = data.formValues[0];
            this.mob.nameTag = name;
        });
    }
    get selectedDestination() {
        const targetId = getProperty(this.mob, "targetId");
        const destinations = this.getDestinations();
        const destination = destinations.find((d)=>d.entity.id == targetId) ?? destinations[0];
        return destination;
    }
    ride(player) {
        if (!this.rocket) return;
        const cost = this.selectedDestination.cost;
        this.rocket.triggerEvent("ns_mvt:locked");
        system.runTimeout(()=>{
            if (this.rocket.getComp("ns_mvt:rocket").ammo < cost) return player.sendMessage("Not enough gunpowder.");
            this.rocket.getComp("ns_mvt:rocket").ammo -= cost;
            this.rocket.getComp("ns_mvt:rocket").destination = this.selectedDestination.entity.loc();
            this.rocket.getComp("ns_mvt:rocket").origin = this.mob.loc();
            this.rocket.getComp("ns_mvt:rocket").ride(player);
            this.rocket.setProperty("ns_mvt:old", true);
            this.playSfx("ns_mvt.rocketstart");
        });
    }
    freeRide(player) {
        if (!this.rocket) return;
        const minCost = 10;
        this.rocket.triggerEvent("ns_mvt:unlocked");
        system.runTimeout(()=>{
            if (this.rocket.getComp("ns_mvt:rocket").ammo < minCost) return player.sendMessage(`You need at least ${minCost} gunpowder.`);
            this.rocket.getComp("ns_mvt:rocket").freeRide(player);
            this.rocket.setProperty("ns_mvt:old", false);
            this.playSfx("ns_mvt.rocketstart");
        });
    }
    dejectRiders() {
        this.mob.getComponent("rideable")?.ejectRiders();
    }
    getRider() {
        return this.mob.getComponent("rideable")?.getRiders()[0];
    }
    constructor(entity){
        super(entity);
        entity.teleport(entity.loc().grid(), {
            rotation: {
                x: 0,
                y: 0
            }
        });
        for(let i = 1; i < 50; i++){
            if (this.mob.nameTag == "" && entity.dimension.getEntities({
                type: "ns_mvt:platform",
                name: "Platform " + i
            }).length == 0) {
                this.mob.nameTag = "Platform " + i;
                break;
            }
        }
        const nearbyPlatforms = entity.dimension.getEntities({
            type: "ns_mvt:platform",
            maxDistance: 0.5,
            location: entity.location
        });
        if (nearbyPlatforms.length > 1) {
            this.pickUp();
        }
    }
}
export { platform as default };
