import { GameMode, ItemStack, system } from "@minecraft/server";
import { comps, isConfigEnabled } from "index";
import { point } from "point";
import mob from "./mob";
import spring from "./spring";
class player extends mob {
    force3rdPersonCamera() {
        this.mob.camera.setCamera("minecraft:third_person");
        this.setProperty("hasForced3rdCamera", true);
    }
    onTick() {
        // const gamemode = GameMode.survival as string;
        const gamemode = this.mob.getGameMode();
        const velocity = point.fromVector3(this.mob.getVelocity());
        const yVelocity = velocity.y;
        const heavyAirTimeWas = this.heavyAirTime;
        const wasInWallMode = this.isInWallMode;
        this.crouchingTime = this.mob.isSneaking ? this.crouchingTime + 1 : 0;
        this.onGroundTime = this.mob.isOnGround ? this.onGroundTime + 1 : 0;
        this.jumpingTime = this.mob.isJumping ? this.jumpingTime + 1 : 0;
        this.sprintingTime = this.mob.isSprinting ? this.sprintingTime + 1 : 0;
        this.airTime = this.mob.isOnGround ? 0 : this.airTime + 1;
        this.fallingTime = yVelocity < 0 ? this.fallingTime + 1 : 0;
        this.heavyAirTime = yVelocity < -0.35 ? this.heavyAirTime + 1 : 0;
        this.viewDirection = point.fromVector3(this.mob.getViewDirection());
        if (this.getProperty("hasForced3rdCamera")) {
            const ride = this.mob.getComponent("riding")?.entityRidingOn;
            if (!ride && this.onGroundTime > 1) {
                this.mob.camera.clear();
                this.setProperty("hasForced3rdCamera", false);
            }
        }
        if (isConfigEnabled("Roll") && !this.isInWallMode && !this.wasWallRunning && heavyAirTimeWas == 4 && this.landedFromCannon && this.mob.isFlying && gamemode != GameMode.spectator && this.sprintingTime) {
            this.queueAnimation("animation.ninja_squirrel_mvt.double_jump", {
                controller: "mvt",
                nextState: "animation.ninja_squirrel_mvt.walk",
                blendOutTime: 0.2
            });
            this.queueAnimation("animation.ninja_squirrel_mvt.jump3", {
                controller: "mvt",
                blendOutTime: 0.2,
                nextState: "animation.ninja_squirrel_mvt.double_jump",
                stopExpression: "q.is_on_ground"
            });
        }
        if (heavyAirTimeWas >= 4 && this.onGroundTime && !this.heavyAirTime && this.mob.isFlying && gamemode != GameMode.spectator && this.sprintingTime) {
            this.mob.dimension.playSound("ns_mvt.land", this.mob.location, {
                volume: 0.5,
                pitch: 1.5
            });
        }
        if (this.onGroundTime) {
            this.onGroundResetValues();
        }
        const wallRunningDirectionWas = this.wallRunningDirection;
        if (!this.isInWallMode) {
            this.climbingYaw = null;
        }
        if (this.isInWallMode || this.isWallRunning) {
            if (this.timeSince(this.wallJumpTimestamp) > 8) {
                this.hasWallJumpedSinceAirTime = false;
            }
        }
        const yRotation = this.mob.getRotation().y;
        this.closeness = (yRotation + 900) % 90;
        if (this.closeness > 45) this.closeness = 90 - this.closeness;
        const springs = Object.values(comps).filter((comp)=>comp instanceof spring);
        for (const spring of springs){
            const inRange = spring.startLoc.getDistance(this.mob.location) < 1;
            if (inRange) {
                if (accessible(spring.mob)) spring.trigger(this.mob);
            }
        }
        if (this.mob.isFlying || gamemode == GameMode.spectator) return;
        if (isConfigEnabled("Dash")) this.dash();
        if (isConfigEnabled("DoubleJump")) this.doubleJump();
        if (!this.isInWallMode && !this.isWallRunning && isConfigEnabled("WaterRun")) {
            this.waterRun();
        }
        let eval1 = 0;
        if (!this.isInWallMode) {
            this.wallMode();
            if (!this.isInWallMode) this.wallRun();
            eval1 = 1;
        } else {
            const rotToClimb = Math.abs(yRotation - this.climbingYaw);
            if (this.closeness < 5 && this.jumpingTime) {
                this.wallMode();
                if (!this.isInWallMode) this.wallRun();
                eval1 = 2;
            } else if (rotToClimb > 30 && this.sprintingTime) {
                this.wallRun();
                if (!this.isWallRunning) this.wallMode();
                eval1 = 3;
            } else {
                this.wallMode();
                if (!this.isInWallMode) this.wallRun();
                eval1 = 4;
            }
        }
        this.wallJump();
        this.log({
            eval1,
            wallRunning: this.isWallRunning,
            wallMode: this.isInWallMode
        });
        this.wallSlide();
        this.dampenFallDamage();
        if (wasInWallMode != this.isInWallMode) {
            if (this.isInWallMode) {
                this.queueAnimation("animation.ninja_squirrel_mvt.climb", {
                    controller: "mvt",
                    blendOutTime: 0.2,
                    nextState: "animation.ninja_squirrel_mvt.walk",
                    stopExpression: "q.is_on_ground"
                });
            }
        } else if (wallRunningDirectionWas != this.wallRunningDirection) {
            if (this.wallRunningDirection == "left") {
                this.queueAnimation("animation.ninja_squirrel_mvt.rot_left", {
                    controller: "mvt",
                    blendOutTime: 0.2,
                    nextState: "animation.ninja_squirrel_mvt.walk",
                    stopExpression: "0"
                });
            } else if (this.wallRunningDirection == "right") {
                this.queueAnimation("animation.ninja_squirrel_mvt.rot_right", {
                    controller: "mvt",
                    blendOutTime: 0.2,
                    nextState: "animation.ninja_squirrel_mvt.walk",
                    stopExpression: "0"
                });
            } else {
                this.queueAnimation("animation.ninja_squirrel_mvt.walk", {
                    controller: "mvt",
                    stopExpression: "0"
                });
            }
        }
        if (!this.isWallRunning) {
            this.wallRunningDirection = "none";
        }
        this.wasWallRunning = this.isWallRunning;
        this.isWallRunning = false;
    }
    onGroundResetValues() {
        this.hasWallRanSinceAirTime = false;
        this.hasDoubleJumpedSinceAirTime = false;
        this.hasDashedSinceAirTime = false;
        this.hasWallJumpedSinceAirTime = false;
        this.landedFromCannon = true;
    }
    dash() {
        if (this.hasDashedSinceAirTime) return;
        if (this.isWallRunning || this.isInWallMode) return;
        if (this.airTime < 2) return;
        if (this.crouchingTime != 1) return;
        const viewDirection = point.fromVector3(this.mob.getViewDirection());
        this.mob.applyKnockback(viewDirection.x, viewDirection.z, 1.4, 0);
        this.mob.addEffect("speed", 5, {
            amplifier: 3,
            showParticles: false
        });
        this.mob.addEffect("slow_falling", 6, {
            amplifier: 1,
            showParticles: false
        });
        this.hasDashedSinceAirTime = true;
        this.queueAnimation("animation.ninja_squirrel_mvt.jump", {
            controller: "mvt",
            blendOutTime: 0.2,
            nextState: "animation.ninja_squirrel_mvt.walk"
        });
        let ticksRan = 0;
        const fnToRun = ()=>{
            this.mob.dimension.spawnParticle("ns_mvt:dash", this.mob.loc().addVector3(viewDirection.multiply(1)));
            this.mob.dimension.spawnParticle("ns_mvt:dash", this.mob.loc().addVector3(viewDirection.multiply(0.5)));
            this.mob.dimension.spawnParticle("ns_mvt:dash", this.mob.loc().addVector3(viewDirection.multiply(-0.5)));
            this.mob.dimension.spawnParticle("ns_mvt:dash", this.mob.loc().addVector3(viewDirection.multiply(-1)));
            if (ticksRan > 2) {
                this.mob.applyKnockback(-viewDirection.x, -viewDirection.z, 0.2, 0);
                return;
            }
            this.mob.applyKnockback(viewDirection.x, viewDirection.z, 2.4, 0);
            ticksRan++;
            system.runTimeout(()=>fnToRun(), 1);
        };
        system.runTimeout(()=>fnToRun());
        this.playSfx("ns_mvt.whoosh2");
        this.addHunger(5, 100);
    }
    addHunger(duration, amplifier) {
        if (!isConfigEnabled("AbilitiesConsumeHunger")) return;
        this.mob.addEffect("hunger", duration, {
            showParticles: false,
            amplifier
        });
    }
    wallJump() {
        if (!isConfigEnabled("WallJump")) return;
        if (!this.isWallRunning) return;
        if (this.airTime < 2) return;
        if (this.jumpingTime != 1) return;
        this.log({
            closeness: this.closeness
        });
        let viewDirection = point.fromVector3(this.mob.getViewDirection());
        if (this.isInWallMode) {
            if (this.closeness < 20) viewDirection = this.climbingRotation.multiply(-1);
        } else if (this.isWallRunning) {
            const left = viewDirection.cross(new point(0, -1, 0)).normalize();
            viewDirection = this.wallRunningDirection == "right" ? left : left.multiply(-1);
        }
        // viewDirection = point.lerp(this.mob.getViewDirection(), viewDirection, 0.7);
        this.wallJumpTimestamp = system.currentTick;
        this.mob.applyKnockback(viewDirection.x, viewDirection.z, 1.8, 0.5);
        this.mob.addEffect("slow_falling", 6, {
            amplifier: 1,
            showParticles: false
        });
        this.hasWallJumpedSinceAirTime = true;
        this.queueAnimation("animation.ninja_squirrel_mvt.jump", {
            controller: "mvt",
            blendOutTime: 0.2,
            nextState: "animation.ninja_squirrel_mvt.walk"
        });
    }
    wallSlide() {
        this.isWallSliding = false;
        if (!this.airTime) return;
        if (this.hasWallJumpedSinceAirTime) return;
        const location = point.fromVector3(this.mob.getHeadLocation());
        const playerViewDirection = point.fromVector3(this.mob.getViewDirection());
        const left = playerViewDirection.cross(new point(0, -0.7, 0)).normalize();
        const blockLeft = this.mob.dimension.getBlock(location.addVector3(left));
        const blockRight = this.mob.dimension.getBlock(location.subtractVector3(left));
        const blockInFront = this.mob.dimension.getBlock(location.addVector3(playerViewDirection.multiply(1.6)));
        // if (blockLeft.isAir && blockRight.isAir && blockInFront.isAir) return;
        if (!this.isSolid(blockLeft.typeId) && !this.isSolid(blockRight.typeId) && !this.isSolid(blockInFront.typeId)) return;
        this.isWallSliding = true;
        if (system.currentTick % 2 == 0) this.mob.addEffect("slow_falling", 2, {
            amplifier: 0,
            showParticles: false
        });
    }
    wallRun() {
        const exit = ()=>{
            this.isWallRunning = false;
            this.wallRunningDirection = "none";
        };
        if (!isConfigEnabled("WallRun")) return exit();
        if (!this.airTime) return exit();
        const headLocation = point.fromVector3(this.mob.getHeadLocation());
        const location = point.fromVector3(this.mob.location);
        const blockBelow1 = this.mob.dimension.getBlock(location.add(0, -1, 0));
        if (this.isSolid(blockBelow1.typeId)) return exit();
        const playerViewDirection = point.fromVector3(this.mob.getViewDirection());
        const left = playerViewDirection.cross(new point(0, -0.5, 0)).normalize();
        const blockLeft = this.mob.dimension.getBlock(headLocation.addVector3(left));
        const blockRight = this.mob.dimension.getBlock(headLocation.subtractVector3(left));
        if (!this.isSolid(blockLeft.typeId) && !this.isSolid(blockRight.typeId)) return exit();
        const wallRunningDirectionWas = this.wallRunningDirection;
        this.wallRunningDirection = !this.isSolid(blockLeft.typeId) ? "right" : "left";
        if (this.hasWallJumpedSinceAirTime && this.wallRunningDirection == wallRunningDirectionWas) return exit();
        const velocity = point.fromVector3(this.mob.getVelocity());
        const xzVelocity = velocity.flat().length();
        this.log({
            u: this.viewDirection.y
        });
        const lookingDir = this.viewDirection.y > 0.3 || this.jumpingTime ? "up" : this.viewDirection.y < -0.3 ? "down" : "middle";
        const verticalForce = lookingDir == "up" ? 0.1 : lookingDir == "down" ? -0.1 : 0.03;
        if (xzVelocity > 0.03 && this.sprintingTime) {
            this.mob.applyKnockback(playerViewDirection.x, playerViewDirection.z, this.sprintingTime ? 0.3 : 0.15, verticalForce);
        } else {
            this.mob.applyKnockback(0, 0, 0, -0.1);
        }
        if (point.getDistance(this.wallRunLocTs, location) > 1.2) {
            this.playSound("step.stone");
            this.wallRunLocTs = location;
        }
        this.isWallRunning = true;
        this.hasWallRanSinceAirTime = true;
    }
    wallMode() {
        if (!isConfigEnabled("Climb")) return;
        const rotation = this.mob.getRotation();
        let snappedRotation = Math.round(rotation.y / 90) * 90;
        if (snappedRotation == -180) snappedRotation = 180;
        let rotationAsVector;
        switch(snappedRotation){
            case 0:
                rotationAsVector = new point(0, 0, 1);
                break;
            case 90:
                rotationAsVector = new point(-1, 0, 0);
                break;
            case -90:
                rotationAsVector = new point(1, 0, 0);
                break;
            case 180:
                rotationAsVector = new point(0, 0, -1);
                break;
            default:
                rotationAsVector = new point(0, 0, 1);
                break;
        }
        if (this.climbingRotation && this.isInWallMode) rotationAsVector = this.climbingRotation;
        const headLocation = point.fromVector3(this.mob.getHeadLocation());
        const blockInViewDir = this.mob.dimension.getBlockFromRay(headLocation, rotationAsVector, {
            maxDistance: 10
        });
        // this.log({ blockInViewDir: blockInViewDir?.block.typeId });
        let blockAbove;
        const playerLocation = point.fromVector3(this.mob.location);
        if (!blockInViewDir?.block || !this.isSolid(blockInViewDir.block.typeId)) {
            if (this.isInWallMode) {
                this.queueAnimation("animation.ninja_squirrel_mvt.walk", {
                    stopExpression: "0",
                    controller: "mvt"
                });
                this.mob.applyKnockback(rotationAsVector.x, rotationAsVector.z, 0.3, 0.7);
            }
            this.isInWallMode = false;
            return;
        }
        const blockLocation = point.fromVector3(blockInViewDir.block.location).floor().addVector3(blockInViewDir.faceLocation);
        const horizontalDistance = blockLocation.subtractVector3(playerLocation).flat().length();
        if (horizontalDistance > 0.8) {
            blockAbove = blockAbove ?? this.mob.dimension.getBlock(playerLocation.add(0, 2.5, 0).addVector3(rotationAsVector.multiply(0.5)));
            if (!this.isSolid(blockAbove?.typeId)) {
                if (this.isInWallMode) {
                    this.queueAnimation("animation.ninja_squirrel_mvt.walk", {
                        stopExpression: "0",
                        controller: "mvt"
                    });
                    this.mob.applyKnockback(rotationAsVector.x, rotationAsVector.z, 0.3, 0.7);
                }
                this.isInWallMode = false;
                return;
            }
        }
        // if (!this.isInWallMode) {
        //     const blockAbove = blockInViewDir.block.above()!;
        //     if (blockAbove.isAir) return;
        // }
        if (this.isInWallMode && this.jumpingTime == 1 || this.onGroundTime) {
            this.isInWallMode = false;
            return;
        }
        if (!this.isInWallMode && !this.jumpingTime) {
            this.isInWallMode = false;
            return;
        }
        if (!this.isInWallMode) {
            this.climbingRotation = rotationAsVector;
            this.climbingYaw = snappedRotation;
        }
        let direction = null;
        let correctDir = false;
        const shadow = false;
        // if (horizontalDistance > 0.45) {
        //     direction = "down";
        // } else if (horizontalDistance < 0.35) {
        //     direction = "up";
        // } else if (this.timeSince(this.wallModeTimestamp) < 3 && this.wallModeLastDirection == "up") {
        //     direction = "up";
        //     shadow = true;
        // } else if (this.timeSince(this.wallModeTimestamp) < 3 && this.wallModeLastDirection == "down") {
        //     direction = "down";
        //     shadow = true;
        // } else if (horizontalDistance > 0.35 && horizontalDistance < 0.65) {
        //     direction = "center";
        // }
        direction = this.jumpingTime ? "up" : "down";
        if (direction == "up") {
            blockAbove = blockAbove ?? this.mob.dimension.getBlock(playerLocation.add(0, 2.5, 0).addVector3(rotationAsVector.multiply(0.5)));
            if (this.isSolid(blockAbove?.typeId)) {
                direction = "back";
            }
        }
        if (horizontalDistance > 0.65) {
            correctDir = true;
        }
        this.log({
            horizontalDistance,
            correctDir,
            direction
        });
        if (direction == "up") {
            this.mob.applyKnockback(rotationAsVector.x, rotationAsVector.z, correctDir ? 0.04 : 0, 0.3);
        } else if (direction == "down") {
            this.mob.applyKnockback(rotationAsVector.x, rotationAsVector.z, correctDir ? 0.04 : 0, -0.2);
        } else if (direction == "back") {
            this.mob.applyKnockback(rotationAsVector.x, rotationAsVector.z, -0.2, 0.1);
        }
        if (!shadow && (direction == "up" || direction == "down")) this.wallModeTimestamp = system.currentTick;
        if (direction) this.wallModeLastDirection = direction;
        if (Math.floor(this.wallModeLastblockLocY / 2) != Math.floor(blockLocation.y / 2)) {
            this.playSound("step.stone");
            this.wallModeLastblockLocY = blockLocation.y;
        }
        this.addHunger(5, 0);
        this.isInWallMode = true;
    }
    doubleJump() {
        if (this.hasWallJumpedSinceAirTime || this.isInWallMode || this.isWallRunning) return;
        if (this.airTime > 3 && !this.hasDoubleJumpedSinceAirTime && this.jumpingTime == 1) {
            this.hasDoubleJumpedSinceAirTime = true;
            const horizontalVelocity = point.fromVector3(this.mob.getVelocity()).horizontal().multiply(1).length();
            const viewDirection = point.fromVector3(this.mob.getViewDirection());
            this.mob.applyKnockback(viewDirection.x, viewDirection.z, horizontalVelocity, 0.7);
            this.mob.applyKnockback(viewDirection.x, viewDirection.z, horizontalVelocity, 0.7);
            this.queueAnimation("animation.ninja_squirrel_mvt.double_jump", {
                controller: "mvt",
                blendOutTime: 0.2,
                nextState: "animation.ninja_squirrel_mvt.walk"
            });
            this.playSfx("ns_mvt.whoosh1");
            this.mob.dimension.spawnParticle("ns_mvt:jump", this.mob.loc().add(0, 0.5, 0));
            this.addHunger(5, 100);
        }
    }
    waterRun() {
        if (!this.sprintingTime) return;
        const block1 = this.mob.dimension.getBlock(this.mob.loc().add(0, 0.05, 0));
        const block1p5 = this.mob.dimension.getBlock(this.mob.loc().add(0, -0.03, 0));
        const block2 = this.mob.dimension.getBlock(this.mob.loc().add(0, -0.15, 0));
        const blockTop = this.mob.dimension.getBlock(this.mob.loc().add(0, 0.6, 0));
        if ((block1?.isLiquid || block2?.isLiquid) && !blockTop?.isLiquid && !this.mob.isSwimming) {
            const viewDirection = point.fromVector3(this.mob.getViewDirection());
            this.mob.applyKnockback(viewDirection.x, viewDirection.z, 0.5, block1p5?.isLiquid ? 0.07 : 0.03);
            const location = this.mob.loc();
            this.mob.dimension.spawnParticle("minecraft:water_splash_particle_manual", location);
            this.mob.dimension.spawnParticle("minecraft:water_splash_particle_manual", location.add(0, 0.2, 0));
            this.mob.dimension.spawnParticle("minecraft:water_splash_particle_manual", location.add(0.2, 0.2, 0.2));
            this.mob.dimension.spawnParticle("minecraft:water_splash_particle_manual", location.add(0.2, 0.2, -0.2));
            this.mob.dimension.spawnParticle("minecraft:water_splash_particle_manual", location.add(-0.2, 0.2, 0.2));
            this.mob.dimension.spawnParticle("minecraft:water_splash_particle_manual", location.add(-0.2, 0.2, -0.2));
            if (system.currentTick % 8 == 0) this.playSound("mob.axolotl.splash");
        }
    }
    dampenFallDamage() {
        if (!isConfigEnabled("Roll")) return;
        if (this.heavyAirTime > 5) return;
        if (this.heavyAirTime > 1 && !this.sprintingTime) return;
        const velocity = point.fromVector3(this.mob.getVelocity());
        if (velocity.y < -0.35) this.mob.addEffect("slow_falling", 1, {
            amplifier: 0,
            showParticles: false
        });
    }
    constructor(entity){
        super(entity);
        this.crouchingTime = 0;
        this.onGroundTime = 0;
        this.jumpingTime = 0;
        this.sprintingTime = 0;
        this.airTime = 0;
        this.fallingTime = 0;
        this.heavyAirTime = 0;
        this.viewDirection = new point(0, 0, 0);
        this.isWallRunning = false;
        this.hasDoubleJumpedSinceAirTime = false;
        this.hasWallRanSinceAirTime = false;
        this.hasDashedSinceAirTime = false;
        this.isInWallMode = false;
        this.climbingRotation = null;
        this.climbingYaw = null;
        this.wallRunningDirection = "none";
        this.hasWallJumpedSinceAirTime = false;
        this.wallModeLastDirection = null;
        this.wallModeTimestamp = 0;
        this.isWallSliding = false;
        this.closeness = 0;
        this.landedFromCannon = true;
        this.wasWallRunning = false;
        this.wallJumpTimestamp = 0;
        this.wallRunLocTs = new point(0, 0, 0);
        this.wallModeLastblockLocY = 0;
        const isInit = entity.getDynamicProperty("isInit");
        if (!isInit) {
            entity.setDynamicProperty("isInit", true);
            const item = new ItemStack("ns_mvt:tutorial");
            entity.dimension.spawnItem(item, entity.location);
        }
        this.queueAnimation("animation.ninja_squirrel_mvt.walk", {
            stopExpression: "0",
            controller: "mvt"
        });
        this.queueAnimation("animation.ninja_squirrel_mvt.cancel_walk", {
            stopExpression: "0",
            controller: "cancel_walk"
        });
        this.runInterval(()=>{
            this.queueAnimation("animation.ninja_squirrel_mvt.cancel_walk", {
                stopExpression: "0",
                controller: "cancel_walk"
            });
            if (!this.isWallRunning && !this.isInWallMode && this.landedFromCannon && this.timeSincePlayedAnimation("animation.ninja_squirrel_mvt.jump") > 15 && this.timeSincePlayedAnimation("animation.ninja_squirrel_mvt.double_jump") > 15 && this.timeSincePlayedAnimation("animation.ninja_squirrel_mvt.jump2") > 50 && this.timeSincePlayedAnimation("animation.ninja_squirrel_mvt.jump3") > 50 && this.timeSincePlayedAnimation("animation.ninja_squirrel_mvt.spring") > 15) {
                this.queueAnimation("animation.ninja_squirrel_mvt.walk", {
                    stopExpression: "0",
                    controller: "mvt"
                });
            }
        }, 50);
    }
}
export { player as default };
