import { system } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";
import { getProperty, setProperty } from "index";
import { point } from "point";
import { util } from "util";
import mob from "./mob";
export function clamp(min, value, max = Infinity) {
    return Math.min(Math.max(min, value), max);
}
class rocket extends mob {
    get origin() {
        if (this.originData) return this.originData;
        const origin = getProperty(this.mob, "origin");
        return origin.length ? point.deserialize(origin) : undefined;
    }
    set origin(value) {
        this.originData = value;
        if (value) this.takenOff = false;
        else this.takenOff = true;
        if (value) setProperty(this.mob, "origin", value.serialize());
        else setProperty(this.mob, "origin", "");
    }
    get destination() {
        if (this.destinationData) return this.destinationData;
        const destination = getProperty(this.mob, "destination");
        return destination.length ? point.deserialize(destination) : undefined;
    }
    set destination(value) {
        this.destinationData = value;
        if (value) setProperty(this.mob, "destination", value.serialize());
        else setProperty(this.mob, "destination", "");
        if (value) {
            const directionToDestination = value.subtractVector3(this.mob.location).normalize();
            const angle = Math.atan2(directionToDestination.x, directionToDestination.z);
            let yRotation = 360 - angle * (180 / Math.PI);
            yRotation = yRotation % 360;
            this.mob.setProperty("ns_mvt:rot_y", 0);
            this.mob.setRotation({
                x: 0,
                y: yRotation
            });
        }
    }
    get takenOff() {
        return getProperty(this.mob, "hasTakenOff");
    }
    set takenOff(value) {
        setProperty(this.mob, "hasTakenOff", value);
    }
    freeRideTick() {
        const player = this.getRider();
        const remainingAmmo = this.ammo;
        player?.onScreenDisplay.setActionBar("§6Remaining Rocket Fuel: §l" + remainingAmmo);
        if (system.currentTick % 40 == 0) {
            this.ammo -= 1;
        }
        if (remainingAmmo <= 0) {
            this.dejectRiders();
            this.mob.clearVelocity();
            this.mob.setRotation({
                x: 0,
                y: 0
            });
            this.mob.setProperty("ns_mvt:rot_x", 0);
            this.setProperty("freeRidingTime", 0);
            return;
        }
        if (!player) {
            this.mob.clearVelocity();
            this.mob.setRotation({
                x: 0,
                y: 0
            });
            this.mob.setProperty("ns_mvt:rot_x", 0);
            this.setProperty("freeRidingTime", 0);
            return;
        }
        this.setProperty("freeRidingTime", this.getProperty("freeRidingTime") + 1);
        if (this.getProperty("freeRidingTime") < 40) {
            const rider = this.getRider();
            rider?.getComp("minecraft:player")?.queueAnimation(`animation.ninja_squirrel_mvt.ride_rocket.${0}`, {
                controller: "ride_rocket",
                stopExpression: "!q.is_riding"
            }, true);
            this.mob.clearVelocity();
            this.mob.applyImpulse({
                x: 0,
                y: 0.1 + this.getProperty("freeRidingTime") * 0.02,
                z: 0
            });
            return;
        }
        let viewDir = point.fromVector3(player.getViewDirection()).add(0, 0.2, 0).normalize();
        // const minYDir = Math.max((100 - this.getProperty("freeRidingTime")) * 0.06, -1);
        // console.warn(viewDir.y, minYDir);
        // viewDir.y = Math.max(minYDir, viewDir.y);
        // console.warn(viewDir.y, minYDir);
        // viewDir = viewDir.normalize();
        // if (viewDir.y < -0.4) {
        //     viewDir = viewDir.multiply(0.6);
        //     const nearbyPlatform = this.mob.dimension.getEntities({
        //         closest: 1,
        //         maxDistance: 1,
        //         location: this.mob.location,
        //         type: "ns_mvt:platform",
        //     })[0];
        //     if (nearbyPlatform) this.mob.teleport(nearbyPlatform.location);
        // }
        const isLookingDown = viewDir.y < -0.3;
        const groundBlock = this.mob.dimension.getBlockFromRay(this.mob.location, new point(0, -1, 0));
        if (groundBlock?.block.isValid()) {
            const heightFromGround = this.mob.location.y - groundBlock.block.location.y;
            if (heightFromGround < 15 && isLookingDown) {
                viewDir = viewDir.multiply(0.7);
            }
            if (heightFromGround < 8 && isLookingDown) {
                viewDir = viewDir.multiply(0.6);
            }
            if (heightFromGround < 3) {
                viewDir = viewDir.multiply(0.6);
            }
        }
        this.mob.clearVelocity();
        this.mob.applyImpulse(viewDir);
        const velocity = point.fromVector3(this.mob.getVelocity());
        const yaw = Math.atan2(velocity.z, velocity.x) * (180 / Math.PI);
        this.mob.setRotation({
            x: 0,
            y: yaw - 90
        });
        let xRotation = clamp(0, velocity.lengthFlat() * 85, 90);
        if (xRotation < 4 || velocity.y < 0 && velocity.length() < 0.5) xRotation = 0;
        this.mob.setProperty("ns_mvt:rot_x", xRotation);
        const rider = this.getRider();
        const xRotRound = Math.max(0, Math.min(90, Math.round(xRotation)));
        if (system.currentTick % 5 == 0) {
            const player = this.getRider();
            this.playSfx("ns_mvt.rocketloop", 0.35, 1.2, player);
        }
        system.runTimeout(()=>{
            if (this.getRider()?.id !== rider?.id) return;
            rider?.getComp("minecraft:player")?.queueAnimation(`animation.ninja_squirrel_mvt.ride_rocket.${xRotRound}`, {
                controller: "ride_rocket",
                stopExpression: "!q.is_riding",
                blendOutTime: 0.2
            }, true);
        }, 1);
    }
    onTick() {
        if (!accessible(this.mob)) return;
        if (this.canBreakBlocks) {
            const goingDownwards = this.mob.getVelocity().y <= 0 && this.destination && point.getDistance(this.mob.location, this.destination) > 2;
            const origLoc = point.fromVector3(this.mob.location);
            const locs = [
                origLoc.add(-0.3, goingDownwards ? -0.7 : 0.7, -0.3),
                origLoc.add(-0.3, goingDownwards ? -0.7 : 0.7, 0.3),
                origLoc.add(0.3, goingDownwards ? -0.7 : 0.7, 0.3),
                origLoc.add(0.3, goingDownwards ? -0.7 : 0.7, 0.3)
            ];
            for (const loc of locs){
                const block = this.mob.dimension.getBlock(loc);
                if (!block?.isAir) {
                    this.mob.dimension.runCommand(`setblock ${loc.x} ${loc.y} ${loc.z} air destroy`);
                    this.blocksBroken++;
                }
            }
        } else {
            this.blocksBroken = 0;
        }
        if (this.blocksBroken > 5) {
            this.dejectRiders();
            this.destination = undefined;
            this.origin = undefined;
            this.blocksBroken = 0;
            this.canBreakBlocks = false;
            return;
        }
        if (this.getProperty("freeRidingTime") > 0) {
            this.freeRideTick();
            return;
        }
        if (!this.destination || !this.origin) {
            this.canBreakBlocks = false;
            return;
        }
        this.canBreakBlocks = true;
        const velocity = point.fromVector3(this.mob.getVelocity());
        const pitch = Math.max(1, velocity.lengthFlat() * 3);
        const int = pitch == 1 ? 10 : 2;
        if (system.currentTick % int == 0) {
            const player = this.getRider();
            this.playSfx("ns_mvt.rocketloop", pitch == 1 ? 1 : 0.35, pitch, player);
        }
        const location = this.mob.location;
        const distanceToDestination = point.getDistance(location, this.destination);
        const distanceToDestinationFlat = point.getDistanceFlat(location, this.destination);
        let direction = new point(0, 1, 0);
        if (velocity.length() > 0.8 || this.takenOff) {
            const dirToDestination = this.destination.subtractVector3(location).normalize();
            direction = dirToDestination;
            direction.y = direction.y + 0.6;
            this.takenOff = true;
            if (distanceToDestinationFlat > 8) {
                direction = direction.setY(0).normalize();
                direction.y = 0.75;
                this.log({
                    num: 1
                });
            } else {
                direction = direction.subtractVector3(velocity).normalize();
                this.log({
                    num: 2
                });
            }
        }
        this.log({
            velocity,
            flat: velocity.length(),
            distanceToDestinationFlat
        });
        this.mob.applyImpulse(direction.scale(0.1));
        if (distanceToDestination < 0.5) {
            this.mob.teleport(this.destination);
            this.destination = undefined;
            this.origin = undefined;
            this.mob.setProperty("ns_mvt:rot_x", 0);
            this.mob.setProperty("ns_mvt:rot_y", 0);
        } else {
            let xRotation = clamp(0, velocity.lengthFlat() * 85, 90);
            if (xRotation < 4 || velocity.y < 0 && velocity.length() < 0.5) xRotation = 0;
            this.mob.setProperty("ns_mvt:rot_x", xRotation);
            const rider = this.getRider();
            // const id = Math.floor(Math.random() * 1000000);
            const xRotRound = Math.max(0, Math.min(90, Math.round(xRotation)));
            rider?.getComp("minecraft:player")?.queueAnimation(`animation.ninja_squirrel_mvt.ride_rocket.${xRotRound}.old`, {
                controller: "ride_rocket",
                stopExpression: "!q.is_riding"
            }, true);
        }
    }
    onInteract(player) {
        const platform = this.platform;
        if (platform) {
            platform.getComp("ns_mvt:platform")?.onInteract(player);
            return;
        }
        const form = new ActionFormData();
        form.title("Rocket");
        const buttons = [];
        buttons.push("Insert gunpowder");
        buttons.push("Pick Up");
        for (const button of buttons){
            form.button(button);
        }
        form.show(player).then((data)=>{
            if (typeof data.selection != "number") return;
            if (!accessible(this.mob)) return;
            switch(buttons[data.selection]){
                case "Pick Up":
                    this.pickUp();
                    break;
                case "Insert gunpowder":
                    this.insertAmmo(player);
                    break;
            }
        });
    }
    get platform() {
        return this.mob.dimension.getEntities({
            closest: 1,
            maxDistance: 1,
            type: "ns_mvt:platform",
            location: this.mob.location
        })[0];
    }
    insertAmmo(player) {
        const ammoInInventoryCount = util.getItemCount(player, "minecraft:gunpowder");
        if (ammoInInventoryCount == 0) {
            player.sendMessage("You need gunpowder in your inventory.");
            return;
        }
        const form = new ModalFormData();
        form.title("Insert Gunpowder");
        form.slider("Amount", 1, ammoInInventoryCount, 1);
        form.show(player).then((data)=>{
            if (!data.formValues) return;
            if (!accessible(this.mob)) return;
            const amount = data.formValues[0];
            const ammoInInventory = util.getItems(player, "minecraft:gunpowder");
            let itemsCountToRemove = amount;
            for (const { itemStack, slot } of ammoInInventory){
                if (itemsCountToRemove <= 0) break;
                if (itemStack.amount <= itemsCountToRemove) {
                    util.setInventoryItem(player, undefined, slot);
                    itemsCountToRemove -= itemStack.amount;
                } else {
                    itemStack.amount -= itemsCountToRemove;
                    util.setInventoryItem(player, itemStack, slot);
                    itemsCountToRemove = 0;
                }
            }
            this.ammo += amount;
        });
    }
    ride(player) {
        const hasRider = this.getRider();
        if (hasRider) return;
        // this.mob.triggerEvent("ns_mvt:locked");
        this.mob.getComponent("rideable")?.addRider(player);
        player.getComp("minecraft:player").force3rdPersonCamera();
    }
    freeRide(player) {
        const hasRider = this.getRider();
        if (hasRider) return;
        this.setProperty("freeRidingTime", 1);
        // this.mob.triggerEvent("ns_mvt:unlocked");
        this.mob.getComponent("rideable")?.addRider(player);
        player.getComp("minecraft:player").force3rdPersonCamera();
    }
    dejectRiders() {
        this.mob.getComponent("rideable")?.ejectRiders();
    }
    getRider() {
        return this.mob.getComponent("rideable")?.getRiders()[0];
    }
    constructor(entity){
        super(entity);
        this.canBreakBlocks = false;
        this.blocksBroken = 0;
        entity.addEffect("slow_falling", 1000000, {
            showParticles: false
        });
    }
}
export { rocket as default };
