import { system } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";
import mob from "./mob";
class spring extends mob {
    trigger(player) {
        if (this.timeSince(this.activeTimestamp) < 20) return;
        this.throw(player);
        player.getComp("minecraft:player").onGroundResetValues();
        this.queueAnimation("animation.ninja_squirrel_mvt.spring.active");
        this.activeTimestamp = system.currentTick;
    }
    throw(player) {
        const xRot = this.mob.getProperty("ns_mvt:rot_x");
        const yRot = 90 - this.mob.getProperty("ns_mvt:rot_y");
        const dir = point.fromRotation(xRot, yRot).add(0, 0.2, 0);
        const xz = dir.flat().length();
        player.getComp("minecraft:player").queueAnimation("animation.ninja_squirrel_mvt.spring", {
            controller: "mvt",
            nextState: "animation.ninja_squirrel_mvt.walk"
        });
        player.applyKnockback(dir.x, dir.z, xz * 1.8, dir.y * 0.7);
        player.applyKnockback(dir.x, dir.z, xz * 1.8, dir.y * 0.7);
        this.playSfx("ns_mvt.spring");
    }
    onInteract(player) {
        const form = new ActionFormData();
        form.title("Spring");
        const buttons = [];
        buttons.push("Set Rotation");
        buttons.push("Pick Up");
        for (const button of buttons){
            form.button(button);
        }
        form.show(player).then((data)=>{
            if (typeof data.selection != "number") return;
            if (!accessible(this.mob)) return;
            switch(buttons[data.selection]){
                case "Pick Up":
                    this.pickUp();
                    break;
                case "Set Rotation":
                    this.setDirection(player);
                    break;
            }
        });
    }
    setDirection(player) {
        const form = new ModalFormData();
        form.title("Set Rotation");
        form.slider("X Rotation", 0, 90, 45, this.mob.getProperty("ns_mvt:rot_x"));
        form.slider("Y Rotation", 0, 360, 45, this.mob.getProperty("ns_mvt:rot_y"));
        form.show(player).then((data)=>{
            if (!data.formValues) return;
            if (!accessible(this.mob)) return;
            const x = data.formValues[0];
            const y = data.formValues[1];
            this.mob.setProperty("ns_mvt:rot_x", x);
            this.mob.setProperty("ns_mvt:rot_y", y);
        });
    }
    constructor(entity){
        super(entity);
        this.activeTimestamp = 0;
        entity.teleport(entity.loc().grid(), {
            rotation: {
                x: 0,
                y: 0
            }
        });
        const nearbySprings = entity.dimension.getEntities({
            type: "ns_mvt:spring",
            maxDistance: 0.5,
            location: entity.location
        });
        if (nearbySprings.length > 1) {
            this.pickUp();
        }
    }
}
export { spring as default };
