export class point {
    static fromVector3(vector) {
        const x = vector.x ?? 0;
        const y = vector.y ?? 0;
        const z = vector.z ?? 0;
        const xSafe = isNaN(x) ? 0 : x;
        const ySafe = isNaN(y) ? 0 : y;
        const zSafe = isNaN(z) ? 0 : z;
        return new point(xSafe, ySafe, zSafe);
    }
    static fromRotation(pitch, yaw) {
        const yawRad = yaw * Math.PI / 180;
        const pitchRad = pitch * Math.PI / 180;
        // Calculate vector components
        const z = Math.sin(pitchRad) * Math.sin(yawRad);
        const y = Math.cos(pitchRad); // Upward/downward component based on pitch
        const x = -Math.sin(pitchRad) * Math.cos(yawRad);
        return new point(x, y, z);
    }
    static fromString(data) {
        const parts = data.split(" ");
        return new point(parseFloat(parts[0]), parseFloat(parts[1]), parseFloat(parts[2]));
    }
    static midpoint(a, b) {
        return new point((a.x + b.x) / 2, (a.y + b.y) / 2, (a.z + b.z) / 2);
    }
    static lerp(a, b, t) {
        return new point(a.x + (b.x - a.x) * t, a.y + (b.y - a.y) * t, a.z + (b.z - a.z) * t);
    }
    serialize() {
        return `${this.x},${this.y},${this.z}`;
    }
    setY(y) {
        return new point(this.x, y, this.z);
    }
    static serialize(point) {
        return `${point.x},${point.y},${point.z}`;
    }
    static deserialize(data) {
        const parts = data.split(",");
        return new point(parseFloat(parts[0]), parseFloat(parts[1]), parseFloat(parts[2]));
    }
    floor() {
        return new point(Math.floor(this.x), Math.floor(this.y), Math.floor(this.z));
    }
    cross(other) {
        return new point(this.y * other.z - this.z * other.y, this.z * other.x - this.x * other.z, this.x * other.y - this.y * other.x);
    }
    horizontal() {
        return new point(this.x, 0, this.z);
    }
    add(x, y, z) {
        return new point(this.x + x, this.y + y, this.z + z);
    }
    grid() {
        return new point(Math.floor(this.x) + 0.5, Math.floor(this.y), Math.floor(this.z) + 0.5);
    }
    normalize() {
        const length = this.length();
        return new point(this.x / length, this.y / length, this.z / length).safe();
    }
    safe() {
        return new point(isNaN(this.x) ? 0 : this.x, isNaN(this.y) ? 0 : this.y, isNaN(this.z) ? 0 : this.z);
    }
    multiply(number) {
        return new point(this.x * number, this.y * number, this.z * number);
    }
    static sumVectors(a, b) {
        return new point(a.x + b.x, a.y + b.y, a.z + b.z);
    }
    length() {
        return Math.sqrt(this.x * this.x + this.y * this.y + this.z * this.z);
    }
    lengthFlat() {
        return Math.sqrt(this.x * this.x + this.z * this.z);
    }
    getDistance(other) {
        return Math.sqrt(Math.pow(this.x - other.x, 2) + Math.pow(this.y - other.y, 2) + Math.pow(this.z - other.z, 2));
    }
    flat() {
        return new point(this.x, 0, this.z);
    }
    addVector3(other) {
        return new point(this.x + other.x, this.y + other.y, this.z + other.z);
    }
    subtractVector3(other) {
        return new point(this.x - other.x, this.y - other.y, this.z - other.z);
    }
    scale(other) {
        return new point(this.x * other, this.y * other, this.z * other);
    }
    static getDistance(a, b) {
        return new point(a.x - b.x, a.y - b.y, a.z - b.z).length();
    }
    static getDistanceFlat(a, b) {
        return new point(a.x - b.x, 0, a.z - b.z).length();
    }
    static vLength(a) {
        return Math.sqrt(a.x * a.x + a.y * a.y + a.z * a.z);
    }
    constructor(x, y, z){
        this.x = x;
        this.y = y;
        this.z = z;
    }
}
