export class util {
    static getInventory(entity) {
        const inventoryComponent = entity.getComponent("minecraft:inventory");
        return inventoryComponent?.container;
    }
    static getEquipment(entity) {
        const equipmentComponent = entity.getComponent("equippable");
        return equipmentComponent;
    }
    static getItemCount(entity, itemTypeId) {
        const items = this.getItems(entity, itemTypeId);
        const count = items.reduce((acc, { itemStack })=>acc + itemStack.amount, 0);
        return count;
    }
    static getItems(entity, itemTypeId) {
        const itemsFound = [];
        const inventory = this.getInventory(entity);
        if (inventory) {
            for(let i = 0; i < inventory.size; i++){
                const itemStack = inventory.getItem(i);
                if (itemStack && itemStack.typeId === itemTypeId) {
                    itemsFound.push({
                        itemStack,
                        slot: i
                    });
                }
            }
        }
        return itemsFound;
    }
    static removeItem(entity, itemTypeId, amount) {
        const ammoInInventory = this.getItems(entity, itemTypeId);
        let itemsCountToRemove = amount;
        for (const { itemStack, slot } of ammoInInventory){
            if (itemsCountToRemove <= 0) break;
            if (itemStack.amount <= itemsCountToRemove) {
                util.setInventoryItem(entity, undefined, slot);
                itemsCountToRemove -= itemStack.amount;
            } else {
                itemStack.amount -= itemsCountToRemove;
                util.setInventoryItem(entity, itemStack, slot);
                itemsCountToRemove = 0;
            }
        }
    }
    static clearEquipmentSlot(entity, slot) {
        const equipment = this.getEquipment(entity);
        if (!equipment) return;
        equipment.setEquipment(slot);
    }
    static setInventoryItem(entity, itemStack, slot) {
        const inventory = this.getInventory(entity);
        if (!inventory) return;
        inventory.setItem(slot, itemStack);
    }
    static setSelectedSlot(player, itemStack) {
        const inventory = this.getInventory(player);
        if (!inventory) return;
        inventory.setItem(player.selectedSlotIndex, itemStack);
    }
    static setEquipmentItem(entity, slot, itemStack) {
        const equipment = this.getEquipment(entity);
        equipment.setEquipment(slot, itemStack);
    }
    static giveItem(entity, itemStack) {
        const inventory = this.getInventory(entity);
        if (!inventory) return;
        inventory.addItem(itemStack);
    }
    static selectedItem(player) {
        const inventory = this.getInventory(player);
        if (!inventory) return;
        const selectedItem = inventory.getItem(player.selectedSlotIndex);
        return selectedItem;
    }
    static emptySlotsCount(entity) {
        const inventory = this.getInventory(entity);
        return inventory?.emptySlotsCount ?? 0;
    }
}
